require 'builder'
require 'httparty'

module NextBuses
	class Request		
		API_URL_STRING = "http://nextbus.mxdata.co.uk/nextbuses/1.0/1"
		
		attr_accessor :stop_code, :username, :password
		
		def initialize(stop_code, username=nil, password=nil)
			@stop_code = stop_code
			@username = username == nil ? NextBuses.configuration.username : username
			@password = password == nil ? NextBuses.configuration.password : password
		end
		
		def execute
			raise NoStopCodeError, 'A stop_code is required' if @stop_code.nil? || @stop_code.empty?
			raise InvalidCredentialsError, 'A username is required' if @username.nil? || @username.empty?
			raise InvalidCredentialsError, 'A password is required' if @password.nil? || @password.empty?
			
			response = HTTParty.post(API_URL_STRING, basic_auth: auth_hash, body: http_body)
			
			services = Array.new
			response['Siri']['ServiceDelivery']['StopMonitoringDelivery']['MonitoredStopVisit'].each do |visit|
				service = Service.new
				service.vehicle_mode = visit['MonitoredVehicleJourney']['VehicleMode']
				service.published_line_name = visit['MonitoredVehicleJourney']['PublishedLineName']
				service.direction_name = visit['MonitoredVehicleJourney']['DirectionName']
				service.operator_ref = visit['MonitoredVehicleJourney']['OperatorRef']
				service.aimed_departure_time = Time.parse(visit['MonitoredVehicleJourney']['MonitoredCall']['AimedDepartureTime'])
				services.push service
			end
			
			services
		end
		
		private
		
		def auth_hash
			{ username: @username, password: @password }
		end
		
		def http_body
			timestamp = Time.now.to_s
			xml = Builder::XmlMarkup.new(indent: 2)
			xml.instruct! :xml, version: '1.0', encoding: 'UTF-8', standalone: 'yes'
			xml.Siri version: '1.0', xmlns: 'http://www.siri.org.uk/' do
				xml.ServiceRequest do |sr|
					sr.RequestTimestamp timestamp
					sr.RequestorRef @username
					sr.StopMonitoringRequest version: 1.0 do
						sr.RequestTimestamp timestamp
						sr.MonitoringRef @stop_code
					end
				end
			end	
		end
	end
	
	class NoStopCodeError < StandardError
	end

	class InvalidCredentialsError < StandardError
	end
end